import json
import os
import pathlib
from typing import Any, cast, Dict, Generator, List, Iterable, Optional, Tuple
from unittest import mock

import pytest
from pyfakefs.fake_filesystem import FakeFilesystem
from click.testing import CliRunner

from gcs_migration.commands.gcs_migration import main
from gcs_migration.commands.identity_mapping_custom import identity_mapping_custom
from gcs_migration.models import MigrationPlan
from gcs_migration.typing.gcs_documents import IdentityMappingDict


@pytest.fixture
def mock_map_identities_environment(
    mock_get_transfer_api: Any,
    mock_get_auth_api: Any,
    migration_plan_cmd_create: MigrationPlan,
    authorization_response_success: Tuple[Optional[str], str, int],
    fs: FakeFilesystem,
) -> Generator[MigrationPlan, None, None]:
    with mock.patch.object(MigrationPlan, "load") as mock_migration_load:
        mock_migration_load.return_value = migration_plan_cmd_create
        migration_plan_cmd_create.root = pathlib.Path(".")
        os.mkdir("migration_plan")
        yield migration_plan_cmd_create


def _mapping_for_domains(domains: Iterable[str]) -> List[IdentityMappingDict]:
    escaped_domains = [d.replace(".", "\\.") for d in domains]
    return [
        {
            "DATA_TYPE": "expression_identity_mapping#1.0.0",
            "mappings": [
                {
                    "source": "{username}",
                    "match": f"(.*)@({'|'.join(escaped_domains)})",
                    "output": "{0}",
                },
            ],
        }
    ]


@pytest.fixture(params=[(None, False), (["other.edu"], False), (["other.edu"], True)])
def mapping_file(
    request: Any,
) -> Optional[Tuple[Optional[str], List[IdentityMappingDict]]]:
    domains, create_file = request.param

    if not domains:
        return None

    mapping = _mapping_for_domains(domains)

    mapfile: Optional[str] = None

    if create_file:
        mapfile = "mapping.json"
        with open(mapfile, "w") as f:
            json.dump(mapping[0], f)

    return (mapfile, mapping)


@pytest.fixture(params=[(None, False), (["other.edu"], False), (["other.edu"], True)])
def bad_mapping_file(
    request: Any,
) -> Optional[Tuple[Optional[str], List[Dict[str, Any]]]]:
    domains, create_file = request.param

    if not domains:
        return None

    mapping = cast(List[Dict[str, Any]], _mapping_for_domains(domains))
    mapping[0]["bogus"] = True

    mapfile: Optional[str] = None

    if create_file:
        mapfile = "mapping.json"
        with open(mapfile, "w") as f:
            json.dump(mapping[0], f)

    return (mapfile, mapping)


@pytest.mark.parametrize("connector", ["file"])
@pytest.mark.parametrize(
    "domains",
    [["example.org", "example.org", "globus.org"], ["example.org", "globus.org"]],
)
def test_identity_mapping_custom(
    fs: FakeFilesystem,
    mock_map_identities_environment: MigrationPlan,
    domains: List[str],
    mapping_file: Optional[Tuple[Optional[str], Optional[List[IdentityMappingDict]]]],
    connector: str,
) -> None:
    runner = CliRunner()
    args: List[str] = []
    migration_plan = mock_map_identities_environment

    for d in domains:
        args.extend(["-d", d])

    if mapping_file:
        mapping_file_path, mappings = mapping_file
    else:
        mapping_file_path, mappings = None, None

    if mapping_file_path:
        args.extend(["-i", mapping_file_path])
    elif mappings:
        for m in mappings:
            args.extend(["-i", json.dumps(m)])

    assert migration_plan.storage_gateway is not None
    assert migration_plan.storage_gateway.allowed_domains is None

    result = runner.invoke(identity_mapping_custom, args, catch_exceptions=False)
    assert result.exit_code == 0

    assert migration_plan.storage_gateway is not None
    assert migration_plan.storage_gateway.allowed_domains is not None

    assert set(domains) == set(migration_plan.storage_gateway.allowed_domains)

    if "-i" in args:
        if mappings:
            expected_mappings = mappings
        else:
            expected_mappings = _mapping_for_domains(domains)

        assert migration_plan.storage_gateway.identity_mappings is not None

        for m in expected_mappings:
            assert m in migration_plan.storage_gateway.identity_mappings

        for m in migration_plan.storage_gateway.identity_mappings:
            assert m in expected_mappings


@pytest.mark.parametrize("connector", ["file"])
@pytest.mark.parametrize(
    "domains",
    [["example.org", "example.org", "globus.org"], ["example.org", "globus.org"]],
)
def test_identity_mapping_custom_bad(
    mock_map_identities_environment: MigrationPlan,
    domains: List[str],
    bad_mapping_file: Optional[
        Tuple[Optional[str], Optional[List[IdentityMappingDict]]]
    ],
    connector: str,
) -> None:
    runner = CliRunner()
    args: List[str] = []
    migration_plan = mock_map_identities_environment

    for d in domains:
        args.extend(["-d", d])

    if bad_mapping_file:
        mapping_file_path, mappings = bad_mapping_file
    else:
        mapping_file_path, mappings = None, None

    if mapping_file_path:
        args.extend(["-i", mapping_file_path])
    elif mappings:
        for m in mappings:
            args.extend(["-i", json.dumps(m)])

    assert migration_plan.storage_gateway is not None
    assert migration_plan.storage_gateway.allowed_domains is None

    result = runner.invoke(identity_mapping_custom, args, catch_exceptions=False)
    if bad_mapping_file:
        assert result.exit_code == 1
        assert migration_plan.storage_gateway is not None
        assert migration_plan.storage_gateway.allowed_domains is None
        assert "Unexpected key" in result.output
    else:
        assert result.exit_code == 0
        assert migration_plan.storage_gateway is not None
        assert migration_plan.storage_gateway.allowed_domains is not None


@pytest.mark.parametrize("connector", ["file"])
@pytest.mark.parametrize(
    "domains",
    [["email.org"], ["unknown.org"]],
)
def test_unusable_domains(
    mock_map_identities_environment: MigrationPlan,
    domains: List[str],
    connector: str,
) -> None:
    runner = CliRunner()
    args: List[str] = []
    migration_plan = mock_map_identities_environment

    for d in domains:
        args.extend(["-d", d])

    assert migration_plan.storage_gateway is not None
    assert migration_plan.storage_gateway.allowed_domains is None

    result = runner.invoke(identity_mapping_custom, args, catch_exceptions=False)
    assert result.exit_code == 1

    assert migration_plan.storage_gateway is not None
    assert migration_plan.storage_gateway.allowed_domains is None
    assert migration_plan.storage_gateway.identity_mappings is None


@pytest.mark.parametrize("connector", ["file"])
def test_requires_logged_in(mock_not_logged_in: None, connector: str) -> None:
    runner = CliRunner()
    result = runner.invoke(
        main, ["identity-mapping", "custom", "--domain", "foo"], catch_exceptions=False
    )
    assert "you will have to login" in result.output
