from typing import Any, Callable, List

import pytest

from gcs_migration.models import (
    BlackPearlMappedCollection,
    PosixMappedCollection,
    MappedCollection,
    Role,
    S3MappedCollection,
)
from gcs_migration.parsers import GridFTPOptionParser
from gcs_migration.typing.transfer_documents import (
    EndpointDocumentDict,
    RoleListDocumentDict,
)

from tests.fixtures.gridftp_config import (
    GridFTPConfigFixture,
    GridFTPConfigFixtureGenerator,
    GridFTPConfigPolicy,
)


@pytest.fixture
def mapped_collection(
    request: Any,
    connector: str,
) -> MappedCollection:
    if connector == "file":
        return request.getfixturevalue("posix_mapped_collection")
    elif connector == "blackpearl":
        return request.getfixturevalue("blackpearl_mapped_collection")
    elif connector == "s3":
        return request.getfixturevalue("s3_mapped_collection")
    else:
        raise AssertionError(
            f"Unsupported connector {connector} for mapped_collection fixture"
        )


@pytest.fixture
def posix_mapped_collection(
    host_endpoint_document: EndpointDocumentDict,
    gridftp_conf: GridFTPConfigPolicy,
    gridftp_config_d: GridFTPConfigFixtureGenerator,
    gridftp_config_gcs: GridFTPConfigPolicy,
    gridftp_config_authorization_myproxy: GridFTPConfigPolicy,
    gridftp_config_credential: GridFTPConfigPolicy,
    gridftp_config_logging: GridFTPConfigPolicy,
    gridftp_config_sharing_policies: GridFTPConfigPolicy,
    role_list_document: RoleListDocumentDict,
    fake_gridftp_config_parser_creator: Callable[
        [GridFTPConfigFixture], GridFTPOptionParser
    ],
    connector: str,
) -> PosixMappedCollection:
    configs = [
        gridftp_config_gcs,
        gridftp_config_authorization_myproxy,
        gridftp_config_credential,
        gridftp_config_logging,
        gridftp_config_sharing_policies,
    ]
    config_dir, config_dir_options = gridftp_config_d(configs)
    config_data = {**gridftp_conf.options, **config_dir_options}
    config_fixture = config_dir, config_data

    gridftp_config = fake_gridftp_config_parser_creator(config_fixture)

    mapped_collection = PosixMappedCollection()
    mapped_collection.gridftp_config = gridftp_config
    mapped_collection.transfer_endpoint = host_endpoint_document
    mapped_collection.role_list_document = role_list_document

    return mapped_collection


@pytest.fixture
def s3_mapped_collection(
    host_endpoint_document: EndpointDocumentDict,
    gridftp_conf: GridFTPConfigPolicy,
    gridftp_config_d: GridFTPConfigFixtureGenerator,
    gridftp_config_gcs: GridFTPConfigPolicy,
    gridftp_config_authorization_myproxy: GridFTPConfigPolicy,
    gridftp_config_credential: GridFTPConfigPolicy,
    gridftp_config_logging: GridFTPConfigPolicy,
    gridftp_config_s3_dsi: GridFTPConfigPolicy,
    role_list_document: RoleListDocumentDict,
    fake_gridftp_config_parser_creator: Callable[
        [GridFTPConfigFixture], GridFTPOptionParser
    ],
    connector: str,
) -> S3MappedCollection:
    configs = [
        gridftp_config_gcs,
        gridftp_config_authorization_myproxy,
        gridftp_config_credential,
        gridftp_config_logging,
        gridftp_config_s3_dsi,
    ]
    config_dir, config_dir_options = gridftp_config_d(configs)
    config_data = {**gridftp_conf.options, **config_dir_options}
    config_fixture = config_dir, config_data

    gridftp_config = fake_gridftp_config_parser_creator(config_fixture)

    mapped_collection = S3MappedCollection()
    mapped_collection.gridftp_config = gridftp_config
    mapped_collection.transfer_endpoint = host_endpoint_document
    mapped_collection.role_list_document = role_list_document

    return mapped_collection


@pytest.fixture
def blackpearl_mapped_collection(
    host_endpoint_document: EndpointDocumentDict,
    gridftp_conf: GridFTPConfigPolicy,
    gridftp_config_d: GridFTPConfigFixtureGenerator,
    gridftp_config_gcs: GridFTPConfigPolicy,
    gridftp_config_authorization_myproxy: GridFTPConfigPolicy,
    gridftp_config_credential: GridFTPConfigPolicy,
    gridftp_config_logging: GridFTPConfigPolicy,
    gridftp_config_blackpearl_dsi_default_path: GridFTPConfigPolicy,
    role_list_document: RoleListDocumentDict,
    fake_gridftp_config_parser_creator: Callable[
        [GridFTPConfigFixture], GridFTPOptionParser
    ],
    connector: str,
) -> BlackPearlMappedCollection:
    configs = [
        gridftp_config_gcs,
        gridftp_config_authorization_myproxy,
        gridftp_config_credential,
        gridftp_config_logging,
        gridftp_config_blackpearl_dsi_default_path,
    ]
    config_dir, config_dir_options = gridftp_config_d(configs)
    config_data = {**gridftp_conf.options, **config_dir_options}
    config_fixture = config_dir, config_data

    gridftp_config = fake_gridftp_config_parser_creator(config_fixture)

    mapped_collection = BlackPearlMappedCollection()
    mapped_collection.gridftp_config = gridftp_config
    mapped_collection.transfer_endpoint = host_endpoint_document
    mapped_collection.role_list_document = role_list_document

    return mapped_collection


@pytest.fixture
def mapped_collection_cmd_create(
    host_endpoint_document: EndpointDocumentDict,
    connector: str,
    gridftp_config_parser: GridFTPOptionParser,
) -> MappedCollection:
    mapped_collection: MappedCollection

    if connector == "file":
        mapped_collection = PosixMappedCollection()
    elif connector == "s3":
        mapped_collection = S3MappedCollection()
    elif connector == "blackpearl":
        mapped_collection = BlackPearlMappedCollection()

    mapped_collection.gridftp_config = gridftp_config_parser
    mapped_collection.transfer_endpoint = host_endpoint_document

    return mapped_collection


@pytest.fixture
def new_roles() -> List[Role]:
    return [
        Role(
            data={
                "id": role_id,
                "principal": principal,
                "principal_type": "identity",
                "role": "administrator",
            }
        )
        for (role_id, principal) in [
            (
                "ffdb05fa-9393-45b7-b893-d2a258a51209",
                "5c395364-9dd9-44bf-a303-0a9b70405753",
            ),
            (
                "31c79afd-bd0b-4a5e-93c1-7d0cc3463826",
                "7f27cc85-fb43-4a94-b705-d73a3e22f901",
            ),
        ]
    ]
