import os
from typing import Any, Callable, Dict, Iterator, List, Optional
from unittest import mock

import pytest
from pyfakefs.fake_filesystem import FakeFilesystem

from gcs_migration.parsers import S3MapEntry, S3MapfileParser


def _s3_map_entry_line(entry: S3MapEntry) -> str:
    if entry.token:
        return ";".join(
            [
                entry.username,
                entry.access_key_id,
                entry.secret_key,
                entry.token,
            ]
        )
    else:
        return ";".join(
            [
                entry.username,
                entry.access_key_id,
                entry.secret_key,
            ]
        )


@pytest.fixture
def s3_map_entry1(sharing_local_user: str) -> S3MapEntry:
    return S3MapEntry(sharing_local_user, "access_key_id1", "secret_key1", None)


@pytest.fixture
def s3_map_entry2() -> S3MapEntry:
    return S3MapEntry("user2", "access_key_id2", "secret_key2", "token2")


@pytest.fixture
def user_without_mapfile() -> str:
    return "user_without_mapfile"


@pytest.fixture
def mock_s3_mapfile(
    s3_map_entry1: S3MapEntry,
    s3_map_entry2: S3MapEntry,
) -> S3MapfileParser:
    res = mock.MagicMock(spec=S3MapfileParser)

    res.map_entries = [s3_map_entry1, s3_map_entry2]
    res.__contains__.side_effect = lambda x: res.map_entries.__contains__(x)
    res.__getitem__.side_effect = lambda x: res.map_entries[x]

    return res


@pytest.fixture
def s3_map_entries(
    s3_map_entry1: S3MapEntry,
    s3_map_entry2: S3MapEntry,
    user_without_mapfile: str,
    s3_process_user: Optional[str],
    mock_users: Callable[[List[str]], Dict[str, Any]],
) -> Iterator[List[S3MapEntry]]:
    users = [s3_map_entry1.username, s3_map_entry2.username, user_without_mapfile]
    if s3_process_user:
        users.append(s3_process_user)

    mock_users(users)

    yield [s3_map_entry1, s3_map_entry2]


@pytest.fixture
def s3_map_files(
    fs: FakeFilesystem,
    s3_map_filename: str,
    s3_map_entries: List[S3MapEntry],
    s3_process_user: Optional[str],
) -> Iterator[List[str]]:
    map_filenames: List[str] = []
    if not s3_process_user and (
        "$HOME" in s3_map_filename or "$LOGNAME" in s3_map_filename
    ):
        for entry in s3_map_entries:
            user_filename = s3_map_filename.replace(
                "$HOME", f"/home/{entry.username}"
            ).replace("$LOGNAME", entry.username)
            fs.makedirs(os.path.dirname(user_filename), exist_ok=True)
            with open(user_filename, "w") as f:
                print(_s3_map_entry_line(entry), file=f)
            map_filenames.append(user_filename)
    else:
        map_filename: str
        if s3_process_user:
            map_filename = s3_map_filename.replace(
                "$HOME", f"/home/{s3_process_user}"
            ).replace("$LOGNAME", s3_process_user)
        else:
            map_filename = s3_map_filename
        fs.makedirs(os.path.dirname(map_filename), exist_ok=True)
        with open(map_filename, "w") as f:
            for entry in s3_map_entries:
                print(_s3_map_entry_line(entry), file=f)
        map_filenames.append(map_filename)

    yield map_filenames
