import os
import pathlib
import pwd

import pytest

from gcs_migration.models import GsiAuthzConfiguration
from gcs_migration.util import AuthorizeSharing


def missing_gss_assist_library() -> bool:
    try:
        import ctypes
        import ctypes.util

        gss_assist = ctypes.CDLL(ctypes.util.find_library("globus_gss_assist"))
        if gss_assist and hasattr(gss_assist, "globus_i_gsi_gss_assist_module"):
            return False
    except Exception:
        pass
    return True


@pytest.mark.skipif(missing_gss_assist_library(), reason="Missing GSS Assist Library")
@pytest.mark.parametrize("mapfile,ok", [('"{0}" {1}', True), ('"{0}X" {1}', False)])
def test_map_and_authorize_user(
    tmp_path: pathlib.Path,
    test_user_cert_subject: str,
    test_user_subject_hash: str,
    test_user_cert_data: str,
    mapfile: str,
    ok: bool,
) -> None:
    gridmap = tmp_path / "grid-mapfile"

    pwent = pwd.getpwuid(os.geteuid())

    with open(gridmap, "w") as f:
        print(mapfile.format(test_user_cert_subject, pwent.pw_name), file=f)

    certdir = tmp_path / "certdir"
    os.makedirs(certdir, mode=0o0755, exist_ok=True)
    with open(certdir / f"{test_user_subject_hash}.0", "w") as f:
        print(test_user_cert_data, file=f)
    with open(certdir / f"{test_user_subject_hash}.signing_policy", "w") as f:
        f.write(
            f"""\
access_id_CA   X509    '{test_user_cert_subject}'
pos_rights     globus  CA:sign
cond_subjects  globus  '"{test_user_cert_subject}"'
"""
        )
    oldcertdir = os.environ.get("X509_CERT_DIR")
    oldgridmap = os.environ.get("GRIDMAP")
    os.environ["X509_CERT_DIR"] = str(certdir)
    os.environ["GRIDMAP"] = str(gridmap)

    gsi_authz_conf = GsiAuthzConfiguration(gridmap=gridmap)
    gsi_authz_conf.callout_dict_list = [None]
    authorize_sharing = AuthorizeSharing(gsi_authz_conf)
    username, subject, idx = authorize_sharing.map_and_authorize_sharing(
        test_user_cert_data
    )
    del os.environ["X509_CERT_DIR"]
    del os.environ["GRIDMAP"]
    if oldcertdir:
        os.environ["X509_CERT_DIR"] = oldcertdir
    if oldgridmap:
        os.environ["GRIDMAP"] = oldgridmap

    if ok:
        assert username == pwent.pw_name
        assert subject == test_user_cert_subject
        assert idx == 0
    else:
        assert username is None
